/* 3dPritner Bed Leveler - Arduino Nano 2018 TechKiwiGradgets
V14 - FIRST RELEASE
*/

//Libraries

// WS2182 LED Driver Library Setup
#include "FastLED.h"
// How many leds in your strip?
#define NUM_LEDS 10 // Note: First LED is address 0
#define DATA_PIN 11 // Note: D11 used to control LED chain

// Define the array of leds
CRGB leds[NUM_LEDS];

//Constants

// Smoothing Variables for LPF
const float alpha = 0.97; 
float smoothedvalue1 = 477; // Tune this value to be the startup baseline value for the LED Bar Graph on powerup

int sthreshold = 10; // Minimum reading from pulseIn function ensure sampling above noise


boolean sample = false; // if true then do not recallibrate thresholds
int calcounter = 0; // Used to count the number of times a sample has been taken
int scount = 200; // Number of reads before calibrating once only after power up
int numsamples = 5; // Number of samples taken before averaging them
int t1 = 100; // Duration in microseconds that LEDs on during callibration

long calav1 = 0; // Used to calcuate average of ten samples

int div1 = 0;// Divisor counter for number of samples taken

// Single Channel denoted by A

int divA = 0;
long tempA = 0;
long pcounterA = 0; // Unfiltered Pulse Width of Channel 1

int pwindow = 2000; // Maximum timeout value for pulsewidth

// ****************************************************
int dly1 = 4; // Period of time that TXR pin stays high (typicLLY 4uS)
int dly2 = 0; // Delay after TXR pin goes LOW before reading starting to read the pulse duration - typically 1 uS 
int dly3 = 2000; // Delay in Microseconds after sampling pulsewidth  before starting next cycle
// ****************************************************

int threshdiff = 15; // Add to average to become specific threshold where LED will light (Changed V21 from 25 to 20)

long pulseav = 0; // Stores value of output for calibration

//Variables

int T1 = 20;// LED Powerup Sequence delay

int baseline = 120; // This is the baseline measurement to the deck
int tbase = 0; //Usedto callibrate the baseline temp variable
int resolution = 1; // Increment value for LED changes
int ytemp = 0;  // Temporary value for LED position on bar graph


void setup() {
  Serial.begin(115200); // Setupserial interface for test data outputs
// WS2182 LED Driver Setup
  LEDS.addLeds<WS2812,DATA_PIN,GRB>(leds,NUM_LEDS); // Default if RGB for this however may vary dependent on LED manufacturer
  LEDS.setBrightness(2); //Set brightness of LEDs here
  // limit my draw to 1A at 5v of power draw
  FastLED.setMaxPowerInVoltsAndMilliamps(5,100);

  FastLED.setDither(0); // Turns off Auto Dithering function to remove flicker


  // Transmit Pulse Channel A
  pinMode(2,OUTPUT); // Pulse output from Arduino                   - D2
  // Receive Pulse Channel A
  pinMode(3,INPUT); // Signal input to Arduino from pin 2 on LM339  - D3  

  
  // initialize the reset button pin 
   pinMode(8, INPUT_PULLUP); // Use pullup feature to emulate an open close command

  
  ledtest(); // startup led test
  
}


void loop() {

// Pulse and read Coil  ---------------------------------------------


// Channel 1 (point of sensor)

digitalWrite(2,HIGH); // Set the TX pin to high
delayMicroseconds(dly1); // Delay before setting low on output pin
digitalWrite(2,LOW); // Set TX pin to low
delayMicroseconds(dly2); // Delay before sampling pulse width

// if value is LOW, pulseIn() waits for the pin to go LOW, starts timing, then waits for the pin to go HIGH and stops timing

pcounterA = pulseIn(3,LOW,pwindow);
digitalWrite(2,LOW); // Set the TX pin to LOW
delayMicroseconds(dly3); // Delay before next pulse 


// Apply Low Pass Filter to signal to smooth Channel 1
  if (pcounterA >= sthreshold) {
    smoothedvalue1 = (alpha * smoothedvalue1) + ( (1 - alpha) * pcounterA);
    }

// Print value then Reset the counter

  Serial.print(pcounterA);
  Serial.print("  ");  
  Serial.print(smoothedvalue1);
  Serial.print("  "); 
  Serial.println(calav1);
 
  
  pcounterA = smoothedvalue1;





// Callibation of Thresholds on Powerup
  // Wait for a period of time to set baseline for each coil
      
   if (sample == false){


    // Set the thresholds

    calav1 = pcounterA;
    
  // Flash LED to show calibrate done

  // 0-3 Blue
    leds[3] = CRGB::Blue;
    FastLED.show();
    delay(t1);  
    leds[3] = CRGB::Black;
 
    leds[2] = CRGB::Blue;
    FastLED.show();
    delay(t1);  
    leds[2] = CRGB::Black;

     leds[1] = CRGB::Blue;
    FastLED.show();
    delay(t1);  
    leds[1] = CRGB::Black;           

     leds[0] = CRGB::Blue;
    FastLED.show();
    delay(t1);  
    leds[0] = CRGB::Black;

  // 3-0 Green
    leds[3] = CRGB::Green;
    FastLED.show();
    delay(t1);  
    leds[3] = CRGB::Black;
 
    leds[2] = CRGB::Green;
    FastLED.show();
    delay(t1);  
    leds[2] = CRGB::Black;

     leds[1] = CRGB::Green;
    FastLED.show();
    delay(t1);  
    leds[1] = CRGB::Black;           

     leds[0] = CRGB::Green;
    FastLED.show();
    delay(t1);  
    leds[0] = CRGB::Black;

    FastLED.show();
    
    // Reset the sample flag
    sample = true;

  }


    // If callibrated use a scale to represent the current position relative to the baseline

    if (sample == true) {
        ytemp = ( calav1 - pcounterA);
    
        // Handle the top end
    
        if (ytemp <= -1){
     
            ytemp = abs(ytemp);
    
            ytemp = ( ytemp / resolution );
    
            if (ytemp > 4) {
    
                 ytemp = 0; // set to the bottom of the scale
            
            } else {
           
            ytemp = ( 4 - ytemp);
            
            }
        } else {
    
            ytemp = ( ytemp / resolution );
    
            if (ytemp > 4) {
    
                 ytemp = 9; // set to the bottom of the scale
            
            } else {
           
            ytemp = ( 4 + ytemp);
            
            }
    
          
        }


    
            // Display a LED in the scale of 10 LEDs addressed 0 to 9
            // REVERSE ACTION ****          
            leds[9-ytemp] = CRGB::Blue;
            FastLED.show();
            delay(T1);   
    
            leds[9-ytemp] = CRGB::Black;
            FastLED.show(); 
    }


        




  

  // Test if push of a button D8 to lock or unlock door
  if(digitalRead(8) == LOW ){ // Check that button pushed
    delay(150); // Debounce by waiting then checking again
      if(digitalRead(8) == LOW){
             sample = false; // Initiate the calibration sequence
             calav1 = 0; // Reset the baseline for calibration
             div1 = 0; // Reset the baseline for calibration
             calcounter = 0;             
        } 
       }     






     


}



void ledtest() { // Powerup sequence for unit

  // Flash LED to show calibrate done

      for (int x = 0; x < 10; x++) {
        leds[x] = CRGB::Blue;
        FastLED.show();
        delay(T1);     
      }
      for (int x = 0; x < 10; x++) {

        leds[x] = CRGB::Black;
        FastLED.show();      
        delay(T1); 
      }

      for (int x = 9; x > -1; x--) {
        leds[x] = CRGB::Green;
        FastLED.show();
        delay(T1);     
      }
      for (int x = 9; x > -1; x--) {

        leds[x] = CRGB::Black;
        FastLED.show();      
        delay(T1); 
      }
      for (int x = 0; x < 10; x++) {
        leds[x] = CRGB::Red;
        FastLED.show();
        delay(T1);     
      }
      for (int x = 0; x < 10; x++) {

        leds[x] = CRGB::Black;
        FastLED.show();      
        delay(T1); 
      }


}




